from typing import Any, List, Optional

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.db.session import get_db
from app.schemas.product import Product as ProductSchema, ProductCreate, ProductUpdate
from app.crud import product

router = APIRouter()


@router.get("", response_model=List[ProductSchema])
def get_products(
    *,
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    search: Optional[str] = None,
    active_only: bool = True
) -> Any:
    """
    Retrieve products.
    Optional filtering by:
    - search: search term in product name
    - active_only: if True, returns only active products
    """
    if search:
        products = product.search_by_name(db, name=search, skip=skip, limit=limit)
    elif active_only:
        products = product.get_active(db, skip=skip, limit=limit)
    else:
        products = product.get_multi(db, skip=skip, limit=limit)
    return products


@router.post("", response_model=ProductSchema, status_code=status.HTTP_201_CREATED)
def create_product(
    *,
    db: Session = Depends(get_db),
    product_in: ProductCreate
) -> Any:
    """
    Create new product.
    """
    # Check if product with the same SKU already exists
    db_product = product.get_by_sku(db, sku=product_in.sku)
    if db_product:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail=f"Product with SKU {product_in.sku} already exists"
        )
    return product.create(db=db, obj_in=product_in)


@router.get("/{product_id}", response_model=ProductSchema)
def get_product(
    *,
    db: Session = Depends(get_db),
    product_id: int
) -> Any:
    """
    Get product by ID.
    """
    db_product = product.get(db=db, id=product_id)
    if not db_product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found"
        )
    return db_product


@router.put("/{product_id}", response_model=ProductSchema)
def update_product(
    *,
    db: Session = Depends(get_db),
    product_id: int,
    product_in: ProductUpdate
) -> Any:
    """
    Update a product.
    """
    db_product = product.get(db=db, id=product_id)
    if not db_product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found"
        )
    
    # If updating SKU, check if the new SKU already exists in another product
    if product_in.sku and product_in.sku != db_product.sku:
        existing_product = product.get_by_sku(db, sku=product_in.sku)
        if existing_product and existing_product.id != product_id:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=f"Product with SKU {product_in.sku} already exists"
            )
    
    return product.update(db=db, db_obj=db_product, obj_in=product_in)


@router.delete("/{product_id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
def delete_product(
    *,
    db: Session = Depends(get_db),
    product_id: int
) -> None:
    """
    Delete a product.
    """
    db_product = product.get(db=db, id=product_id)
    if not db_product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=f"Product with ID {product_id} not found"
        )
    product.remove(db=db, id=product_id)
    return None